@@ -30,18 +30,18 @@
 
 #include "libcineon/Cineon.h"
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "imageio.h"
-#include "fmath.h"
-#include "strutil.h"
+#include <OpenImageIO/dassert.h>
+#include <OpenImageIO/typedesc.h>
+#include <OpenImageIO/imageio.h>
+#include <OpenImageIO/fmath.h>
+#include <OpenImageIO/strutil.h>
 
 using namespace cineon;
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 
-class CineonInput : public ImageInput {
+class CineonInput final : public ImageInput {
 public:
     CineonInput () : m_stream(NULL) { init(); }
     virtual ~CineonInput () { close(); }
@@ -80,6 +80,8 @@ OIIO_EXPORT ImageInput *cineon_input_imageio_create () { return new CineonInput;
 
 OIIO_EXPORT int cineon_imageio_version = OIIO_PLUGIN_VERSION;
 
+OIIO_EXPORT const char* cineon_imageio_library_version () { return NULL; }
+
 OIIO_EXPORT const char * cineon_input_extensions[] = {
     "cin", NULL
 };
@@ -139,7 +141,7 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
                     std::string ch = Strutil::format ("I%d", gscount);
                     m_spec.channelnames.push_back (ch);
                 } else
-                    m_spec.channelnames.push_back ("I");
+                    m_spec.channelnames.emplace_back("I");
                 break;
             case cineon::kPrintingDensityRed:
             case cineon::kRec709Red:
@@ -147,7 +149,7 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
                     std::string ch = Strutil::format ("R%d", rcount);
                     m_spec.channelnames.push_back (ch);
                 } else
-                    m_spec.channelnames.push_back ("R");
+                    m_spec.channelnames.emplace_back("R");
                 break;
             case cineon::kPrintingDensityGreen:
             case cineon::kRec709Green:
@@ -155,7 +157,7 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
                     std::string ch = Strutil::format ("G%d", gcount);
                     m_spec.channelnames.push_back (ch);
                 } else
-                    m_spec.channelnames.push_back ("G");
+                    m_spec.channelnames.emplace_back("G");
                 break;
             case cineon::kPrintingDensityBlue:
             case cineon::kRec709Blue:
@@ -163,7 +165,7 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
                     std::string ch = Strutil::format ("B%d", bcount);
                     m_spec.channelnames.push_back (ch);
                 } else
-                    m_spec.channelnames.push_back ("B");
+                    m_spec.channelnames.emplace_back("B");
                 break;
             default:
                 std::string ch = Strutil::format ("channel%d", (int)m_spec.channelnames.size());
@@ -206,6 +208,12 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
     }
     m_spec.attribute ("Orientation", orientation);
 
+#if 1
+    // This is not very smart, but it seems that as a practical matter,
+    // all Cineon files are log. So ignore the gamma field and just set
+    // the color space to KodakLog.
+    m_spec.attribute ("oiio:ColorSpace", "KodakLog");
+#else
     // image linearity
     // FIXME: making this more robust would require the per-channel transfer
     // function functionality which isn't yet in OIIO
@@ -216,15 +224,17 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
             m_spec.attribute ("oiio:ColorSpace", "Rec709");
         default:
             // either grayscale or printing density
-            if (!isinf (m_cin.header.Gamma ()))
+            if (!isinf (m_cin.header.Gamma ()) && m_cin.header.Gamma () != 0.0f)
                 // actual gamma value is read later on
-                m_spec.attribute ("oiio:ColorSpace", "GammaCorrected");
+                m_spec.attribute ("oiio:ColorSpace",
+                                  Strutil::format("GammaCorrected%.2g", g));
             break;
     }
 
     // gamma exponent
-    if (!isinf (m_cin.header.Gamma ()))
+    if (!isinf (m_cin.header.Gamma ()) && m_cin.header.Gamma () != 0.0f)
         m_spec.attribute ("oiio:Gamma", (float) m_cin.header.Gamma ());
+#endif
 
     // general metadata
     // some non-compliant writers will dump a field filled with 0xFF rather
@@ -266,10 +276,11 @@ CineonInput::open (const std::string &name, ImageSpec &newspec)
                                             CINEON_SET_ATTRIB(x, )
 #define CINEON_SET_ATTRIB_COORDS(x)     m_cin.header.x (floats);            \
                                         if (!isinf (floats[0])              \
-                                            && !isinf (floats[1]))          \
+                                            && !isinf (floats[1])           \
+                                            && !(floats[0] == 0. && floats[1] == 0.)) \
                                             m_spec.attribute ("cineon:" #x, \
-                                                TypeDesc (TypeDesc::FLOAT,  \
-                                                2), &floats[0])
+                                                TypeDesc (TypeDesc::FLOAT, 2), \
+                                                &floats[0])
 #define CINEON_SET_ATTRIB_STR(X, x)     if (m_cin.header.x[0]               \
                                         && m_cin.header.x[0] != char(-1))   \
                                             m_spec.attribute ("cineon:" #X, \
