@@ -124,6 +124,15 @@ inline int receive_frame(AVCodecContext *avctx, AVFrame *picture,
 }
 #endif
 
+
+// Changes for ffmpeg 4.0
+#define USE_FFMPEG_4_0 (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(58, 18, 100))
+
+#if (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(56, 56, 100))
+#  define CODEC_CAP_DELAY AV_CODEC_CAP_DELAY
+#endif
+
+
 #include <boost/thread/once.hpp>
 
 #include <OpenImageIO/imageio.h>
