@@ -194,8 +194,8 @@ private:
 
         PartInfo () : initialized(false) { }
         ~PartInfo () { }
-        void parse_header (const Imf::Header *header);
-        void query_channels (const Imf::Header *header);
+        bool parse_header (OpenEXRInput *in, const Imf::Header *header);
+        bool query_channels (OpenEXRInput *in, const Imf::Header *header);
     };
 
     std::vector<PartInfo> m_parts;        ///< Image parts
@@ -343,7 +343,12 @@ OpenEXRInput::OpenEXRInput ()
 bool
 OpenEXRInput::valid_file (const std::string &filename) const
 {
-    return Imf::isOpenExrFile (filename.c_str());
+    try {
+        OpenEXRInputStream IStream (filename.c_str());
+        return Imf::isOpenExrFile (IStream);
+    } catch (const std::exception &e) {
+        return false;
+    }
 }
 
 
@@ -356,8 +361,7 @@ OpenEXRInput::open (const std::string &name, ImageSpec &newspec)
         error ("Could not open file \"%s\"", name.c_str());
         return false;
     }
-    bool tiled;
-    if (! Imf::isOpenExrFile (name.c_str(), tiled)) {
+    if (! valid_file(name)) {
         error ("\"%s\" is not an OpenEXR file", name.c_str());
         return false;
     }
@@ -419,11 +423,13 @@ numlevels (int width, int roundingmode)
 
 
 
-void
-OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
+bool
+OpenEXRInput::PartInfo::parse_header (OpenEXRInput *in,
+                                      const Imf::Header *header)
 {
+    bool ok = true;
     if (initialized)
-        return;
+        return ok;
 
     ASSERT (header);
     spec = ImageSpec();
@@ -465,7 +471,8 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
         levelmode = Imf::ONE_LEVEL;
         nmiplevels = 1;
     }
-    query_channels (header);   // also sets format
+    if (! query_channels (in, header))   // also sets format
+        return false;
 
     spec.deep = Strutil::istarts_with (header->type(), "deep");
 
@@ -523,8 +530,7 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
             spec.attribute ("compression", comp);
     }
 
-    for (Imf::Header::ConstIterator hit = header->begin();
-             hit != header->end();  ++hit) {
+    for (auto hit = header->begin(); hit != header->end();  ++hit) {
         const Imf::IntAttribute *iattr;
         const Imf::FloatAttribute *fattr;
         const Imf::StringAttribute *sattr;
@@ -707,6 +713,7 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
     pvt::check_texture_metadata_sanity (spec);
 
     initialized = true;
+    return ok;
 }
 
 
@@ -736,10 +743,13 @@ struct ChanNameHolder {
     int special_index;
     Imf::PixelType exr_data_type;
     TypeDesc datatype;
+    int xSampling;
+    int ySampling;
 
-    ChanNameHolder (string_view fullname, int n, Imf::PixelType exrtype)
-        : fullname(fullname), exr_channel_number(n), exr_data_type(exrtype),
-          datatype(TypeDesc_from_ImfPixelType(exrtype))
+    ChanNameHolder (string_view fullname, int n, const Imf::Channel &exrchan)
+        : fullname(fullname), exr_channel_number(n), exr_data_type(exrchan.type),
+          datatype(TypeDesc_from_ImfPixelType(exrchan.type)),
+          xSampling(exrchan.xSampling), ySampling(exrchan.ySampling)
     {
         size_t dot = fullname.find_last_of ('.');
         if (dot == string_view::npos) {
@@ -780,20 +790,20 @@ struct ChanNameHolder {
 
 
 
-void
-OpenEXRInput::PartInfo::query_channels (const Imf::Header *header)
+bool
+OpenEXRInput::PartInfo::query_channels (OpenEXRInput *in,
+                                        const Imf::Header *header)
 {
     ASSERT (! initialized);
+    bool ok = true;
     spec.nchannels = 0;
     const Imf::ChannelList &channels (header->channels());
     std::vector<std::string> channelnames;  // Order of channels in file
     std::vector<ChanNameHolder> cnh;
     int c = 0;
-    for (Imf::ChannelList::ConstIterator ci = channels.begin();
-         ci != channels.end();  ++c, ++ci) {
-        cnh.emplace_back (ci.name(), c, ci.channel().type);
-        ++spec.nchannels;
-    }
+    for (auto ci = channels.begin(); ci != channels.end();  ++c, ++ci)
+        cnh.emplace_back (ci.name(), c, ci.channel());
+    spec.nchannels = int(cnh.size());
     std::sort (cnh.begin(), cnh.end(), ChanNameHolder::compare_cnh);
     // Now we should have cnh sorted into the order that we want to present
     // to the OIIO client.
@@ -813,11 +823,18 @@ OpenEXRInput::PartInfo::query_channels (const Imf::Header *header)
         if (spec.z_channel < 0 && (Strutil::iequals (cnh[c].suffix, "Z") ||
                                    Strutil::iequals (cnh[c].suffix, "Depth")))
             spec.z_channel = c;
+        if (cnh[c].xSampling != 1 || cnh[c].ySampling != 1) {
+            ok = false;
+            in->error ("Subsampled channels are not supported (channel \"%s\" has sampling %d,%d).",
+                       cnh[c].fullname, cnh[c].xSampling, cnh[c].ySampling);
+            // FIXME: Some day, we should handle channel subsampling.
+        }
     }
     ASSERT ((int)spec.channelnames.size() == spec.nchannels);
     ASSERT (spec.format != TypeDesc::UNKNOWN);
     if (all_one_format)
         spec.channelformats.clear();
+    return ok;
 }
 
 
@@ -838,7 +855,8 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
         const Imf::Header *header = NULL;
         if (m_input_multipart)
             header = &(m_input_multipart->header(subimage));
-        part.parse_header (header);
+        if (! part.parse_header (this, header))
+            return false;
         part.initialized = true;
     }
 
