@@ -38,15 +38,18 @@
 
 
 // This plugin utilises LibRaw:
-// http://www.libraw.org/
-// Documentation: 
-// http://www.libraw.org/docs
+//   http://www.libraw.org/
+// Documentation:
+//   http://www.libraw.org/docs
+// Example raw images from many camera models:
+//   https://www.rawsamples.ch
+
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 class RawInput final : public ImageInput {
 public:
-    RawInput () : m_process(true), m_unpacked(false), m_image(NULL) {}
+    RawInput () {}
     virtual ~RawInput() { close(); }
     virtual const char * format_name (void) const { return "raw"; }
     virtual int supports (string_view feature) const {
@@ -61,14 +64,19 @@ public:
 
 private:
     bool process();
-    bool m_process;
+    bool m_process = true;
     bool m_unpacked = false;
-    LibRaw m_processor;
-    libraw_processed_image_t *m_image;
+    std::unique_ptr<LibRaw> m_processor;
+    libraw_processed_image_t *m_image = nullptr;
     std::string m_filename;
+    ImageSpec m_config;  // save config requests
 
     bool do_unpack();
     void read_tiff_metadata (const std::string &filename);
+
+    // Do the actual open. It expects m_filename and m_config to be set.
+    bool open_raw (bool unpack, const std::string &name,
+                   const ImageSpec &config);
 };
 
 
@@ -110,54 +118,72 @@ bool
 RawInput::open (const std::string &name, ImageSpec &newspec,
                 const ImageSpec &config)
 {
-    int ret;
+    m_filename = name;
+    m_config = config;
+
+    // For a fresh open, we are concerned with just reading all the
+    // meatadata quickly, because maybe that's all that will be needed. So
+    // call open_raw passing unpack=false. This will not read the pixels! We
+    // will need to close and re-open with unpack=true if and when we need
+    // the actual pixel values.
+    bool ok = open_raw (false, m_filename, m_config);
+    if (ok)
+        newspec = m_spec;
+    return ok;
+}
+
+
+
+bool
+RawInput::open_raw (bool unpack, const std::string &name,
+                    const ImageSpec &config)
+{
+    // std::cout << "open_raw " << name << " unpack=" << unpack << "\n";
+    ASSERT (! m_processor);
+    m_processor.reset (new LibRaw);
 
     // open the image
     m_filename = name;
-    if ( (ret = m_processor.open_file(name.c_str()) ) != LIBRAW_SUCCESS) {
-        error ("Could not open file \"%s\", %s", name.c_str(), libraw_strerror(ret));
+    int ret;
+    if ( (ret = m_processor->open_file(name.c_str()) ) != LIBRAW_SUCCESS) {
+        error ("Could not open file \"%s\", %s", m_filename, libraw_strerror(ret));
         return false;
     }
 
-#if 0
-    // We used to unpack here, but that is needlessly expensive for an app
-    // that is only opening the file to read the metadata. So we now do it
-    // upon the first scanline read.
-
-    // We also no longer call adjust_sizes_info_only(), I think that doesn't
-    // hurt us in any practical sense, but be on the lookout for trouble.
-    if (! do_unpack())
-        return false;
-
-    // Forcing the Libraw to adjust sizes based on the capture device orientation
-    m_processor.adjust_sizes_info_only();
-#endif
+    ASSERT (!m_unpacked);
+    if (unpack) {
+        if ( (ret = m_processor->unpack() ) != LIBRAW_SUCCESS) {
+            error ("Could not unpack \"%s\", %s", m_filename, libraw_strerror(ret));
+            return false;
+        }
+    }
+    m_processor->adjust_sizes_info_only();
 
     // Set file information
-    m_spec = ImageSpec(m_processor.imgdata.sizes.iwidth,
-                       m_processor.imgdata.sizes.iheight,
+    m_spec = ImageSpec(m_processor->imgdata.sizes.iwidth,
+                       m_processor->imgdata.sizes.iheight,
                        3, // LibRaw should only give us 3 channels
                        TypeDesc::UINT16);
 
     // Output 16 bit images
-    m_processor.imgdata.params.output_bps = 16;
+    m_processor->imgdata.params.output_bps = 16;
 
     // Set the gamma curve to Linear
     m_spec.attribute("oiio:ColorSpace","Linear");
-    m_processor.imgdata.params.gamm[0] = 1.0;
-    m_processor.imgdata.params.gamm[1] = 1.0;
+    m_processor->imgdata.params.gamm[0] = 1.0;
+    m_processor->imgdata.params.gamm[1] = 1.0;
 
     // Disable exposure correction (unless config "raw:auto_bright" == 1)
-    m_processor.imgdata.params.no_auto_bright =
+    m_processor->imgdata.params.no_auto_bright =
         ! config.get_int_attribute("raw:auto_bright", 0);
     // Use camera white balance if "raw:use_camera_wb" is not 0
-    m_processor.imgdata.params.use_camera_wb =
+    m_processor->imgdata.params.use_camera_wb =
         config.get_int_attribute("raw:use_camera_wb", 1);
     // Turn off maximum threshold value (unless set to non-zero)
-    m_processor.imgdata.params.adjust_maximum_thr =
+    m_processor->imgdata.params.adjust_maximum_thr =
         config.get_float_attribute("raw:adjust_maximum_thr", 0.0f);
     // Set camera maximum value if "raw:user_sat" is not 0
-    m_processor.imgdata.params.user_sat =
+    m_processor->imgdata.params.user_sat =
         config.get_int_attribute("raw:user_sat", 0);
 
     // Use embedded color profile. Values mean:
@@ -166,7 +192,7 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
     //    (always), for other files only if use_camera_wb is set.
     // 3: use embedded color data (if present) regardless of white
     //    balance setting.
-    m_processor.imgdata.params.use_camera_matrix =
+    m_processor->imgdata.params.use_camera_matrix =
         config.get_int_attribute("raw:use_camera_matrix", 1);
 
 
@@ -190,7 +216,7 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
             if (Strutil::iequals (cs, colorspaces[c]))
                 break;
         if (colorspaces[c])
-            m_processor.imgdata.params.output_color = c;
+            m_processor->imgdata.params.output_color = c;
         else {
 #if LIBRAW_VERSION < LIBRAW_MAKE_VERSION(0,18,0)
             if (cs == "ACES")
@@ -206,7 +232,7 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
         m_spec.attribute("raw:ColorSpace", cs);
     } else {
         // By default we use sRGB primaries for simplicity
-        m_processor.imgdata.params.output_color = 1;
+        m_processor->imgdata.params.output_color = 1;
         m_spec.attribute("raw:ColorSpace", "sRGB");
     }
 
@@ -217,12 +243,24 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
             error("raw:Exposure invalid value. range 0.25f - 8.0f");
             return false;
         }
-        m_processor.imgdata.params.exp_correc = 1; // enable exposure correction
-        m_processor.imgdata.params.exp_shift = exposure; // set exposure correction
+        m_processor->imgdata.params.exp_correc = 1; // enable exposure correction
+        m_processor->imgdata.params.exp_shift = exposure; // set exposure correction
         // Set the attribute in the output spec
         m_spec.attribute ("raw:Exposure", exposure);
     }
 
+    // Highlight adjustment
+    int highlight_mode = config.get_int_attribute("raw:HighlightMode", 0);
+    if (highlight_mode != 0)
+    {
+        if (highlight_mode < 0 || highlight_mode > 9) {
+            error("raw:HighlightMode invalid value. range 0-9");
+            return false;
+        }
+        m_processor->imgdata.params.highlight = highlight_mode;
+        m_spec.attribute ("raw:HighlightMode", highlight_mode);
+    }
+
     // Interpolation quality
     // note: LibRaw must be compiled with demosaic pack GPL2 to use demosaic
     // algorithms 5-9. It must be compiled with demosaic pack GPL3 for
@@ -253,12 +291,12 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
             if (Strutil::iequals (demosaic, demosaic_algs[d]))
                 break;
         if (demosaic_algs[d])
-            m_processor.imgdata.params.user_qual = d;
+            m_processor->imgdata.params.user_qual = d;
         else if (Strutil::iequals (demosaic, "none")) {
 #ifdef LIBRAW_DECODER_FLATFIELD
             // See if we can access the Bayer patterned data for this raw file
             libraw_decoder_info_t decoder_info;
-            m_processor.get_decoder_info(&decoder_info);
+            m_processor->get_decoder_info(&decoder_info);
             if (!(decoder_info.decoder_flags & LIBRAW_DECODER_FLATFIELD)) {
                 error("Unable to extract unbayered data from file \"%s\"", name.c_str());
                 return false;
@@ -268,13 +306,10 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
             // User has selected no demosaicing, so no processing needs to be done
             m_process = false;
 
-            // The image width and height may be different now, so update with new values
-            // Also we will only be reading back a single, bayered channel
-            m_spec.width = m_processor.imgdata.sizes.raw_width;
-            m_spec.height = m_processor.imgdata.sizes.raw_height;
+            // This will read back a single, bayered channel
             m_spec.nchannels = 1;
             m_spec.channelnames.clear();
-            m_spec.channelnames.emplace_back("R");
+            m_spec.channelnames.emplace_back("Y");
 
             // Also, any previously set demosaicing options are void, so remove them
             m_spec.erase_attribute("oiio:Colorspace");
@@ -288,38 +323,38 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
         // Set the attribute in the output spec
         m_spec.attribute("raw:Demosaic", demosaic);
     } else {
-        m_processor.imgdata.params.user_qual = 3;
+        m_processor->imgdata.params.user_qual = 3;
         m_spec.attribute("raw:Demosaic", "AHD");
     }
 
     // Metadata
 
-    const libraw_image_sizes_t &sizes (m_processor.imgdata.sizes);
+    const libraw_image_sizes_t &sizes (m_processor->imgdata.sizes);
     m_spec.attribute ("PixelAspectRatio", (float)sizes.pixel_aspect);
-    // FIXME: sizes. top_margin, left_margin, raw_pitch, flip, mask?
+    // FIXME: sizes. top_margin, left_margin, raw_pitch, mask?
 
-    const libraw_iparams_t &idata (m_processor.imgdata.idata);
+    const libraw_iparams_t &idata (m_processor->imgdata.idata);
     if (idata.make[0])
         m_spec.attribute ("Make", idata.make);
     if (idata.model[0])
         m_spec.attribute ("Model", idata.model);
     // FIXME: idata. dng_version, is_foveon, colors, filters, cdesc
 
-    const libraw_colordata_t &color (m_processor.imgdata.color);
+    const libraw_colordata_t &color (m_processor->imgdata.color);
     m_spec.attribute("Exif:Flash", (int) color.flash_used);
     if (color.model2[0])
         m_spec.attribute ("Software", color.model2);
 
     // FIXME -- all sorts of things in this struct
 
-    const libraw_imgother_t &other (m_processor.imgdata.other);
+    const libraw_imgother_t &other (m_processor->imgdata.other);
     m_spec.attribute ("Exif:ISOSpeedRatings", (int) other.iso_speed);
     m_spec.attribute ("ExposureTime", other.shutter);
     m_spec.attribute ("Exif:ShutterSpeedValue", -log2f(other.shutter));
     m_spec.attribute ("FNumber", other.aperture);
     m_spec.attribute ("Exif:ApertureValue", 2.0f * log2f(other.aperture));
     m_spec.attribute ("Exif:FocalLength", other.focal_len);
-    struct tm * m_tm = localtime(&m_processor.imgdata.other.timestamp);
+    struct tm * m_tm = localtime(&m_processor->imgdata.other.timestamp);
     char datetime[20];
     strftime (datetime, 20, "%Y-%m-%d %H:%M:%S", m_tm);
     m_spec.attribute ("DateTime", datetime);
@@ -330,12 +365,16 @@ RawInput::open (const std::string &name, ImageSpec &newspec,
     if (other.artist[0])
         m_spec.attribute ("Artist", other.artist);
 
-    // FIXME -- thumbnail possibly in m_processor.imgdata.thumbnail
+    // libraw reoriented the image for us, so squash any orientation
+    // metadata we may have found in the Exif. Preserve the original as
+    // "raw:Orientation".
+    m_spec.attribute ("Orientation", m_spec.get_int_attribute("Orientation", 1));
+    m_spec.attribute ("Orientation", 1);
+
+    // FIXME -- thumbnail possibly in m_processor->imgdata.thumbnail
 
     read_tiff_metadata (name);
 
-    // Copy the spec to return to the user
-    newspec = m_spec;
     return true;
 }
 
@@ -374,10 +413,11 @@ RawInput::close()
 {
     if (m_image) {
         LibRaw::dcraw_clear_mem(m_image);
-        m_image = NULL;
-        m_unpacked = false;
-        m_process = true;
+        m_image = nullptr;
     }
+    m_processor.reset ();
+    m_unpacked = false;
+    m_process = true;
     return true;
 }
 
@@ -388,13 +428,13 @@ RawInput::do_unpack ()
 {
     if (m_unpacked)
         return true;
-    int ret;
-    if ( (ret = m_processor.unpack() ) != LIBRAW_SUCCESS) {
-        error ("Could not unpack \"%s\", %s", m_filename, libraw_strerror(ret));
-        return false;
-    }
+
+    // We need to unpack but we didn't when we opened the file. Close and
+    // re-open with unpack.
+    close ();
+    bool ok = open_raw (true, m_filename, m_config);
     m_unpacked = true;
-    return true;
+    return ok;
 }
 
 
@@ -403,13 +443,13 @@ bool
 RawInput::process()
 {
     if (!m_image) {
-        int ret = m_processor.dcraw_process();
+        int ret = m_processor->dcraw_process();
         if (ret != LIBRAW_SUCCESS) {
             error("Processing image failed, %s", libraw_strerror(ret));
             return false;
         }
 
-        m_image = m_processor.dcraw_make_mem_image(&ret);
+        m_image = m_processor->dcraw_make_mem_image(&ret);
         if (!m_image) {
             error("LibRaw failed to create in memory image");
             return false;
@@ -443,7 +483,7 @@ RawInput::read_native_scanline (int y, int z, void *data)
     if (! m_process) {
         // The user has selected not to apply any debayering.
         // We take the raw data directly
-        unsigned short *scanline = &((m_processor.imgdata.rawdata.raw_image)[m_spec.width*y]);
+        unsigned short *scanline = &((m_processor->imgdata.rawdata.raw_image)[m_spec.width*y]);
         memcpy(data, scanline, m_spec.scanline_bytes(true));
         return true;
     }
